@@ -1,3 +1,4 @@
+from tkinter import N
 import torch
 from typing import List, Tuple
 from pymed import PubMed 
@@ -18,6 +19,8 @@ import copy
 from Bio import Entrez, Medline
 from pyvis.network import Network
 import re
+import random
+import numpy as np
 
 import nltk
 nltk.download('punkt')
@@ -25,13 +28,20 @@ nltk.download('punkt')
 from transformers import AutoTokenizer, AutoModel, AutoModelForSequenceClassification
 from transformers import pipeline
 
+chemprot_model = AutoModelForSequenceClassification.from_pretrained(
+    "pier297/autotrain-chemprot-re-838426740")
+chemprot_tokenizer = AutoTokenizer.from_pretrained(
+    "pier297/autotrain-chemprot-re-838426740")
+
 rel_tokenizer = AutoTokenizer.from_pretrained("JacopoBandoni/BioBertRelationGenesDiseases")
 rel_model = AutoModelForSequenceClassification.from_pretrained("JacopoBandoni/BioBertRelationGenesDiseases")
 
+chemprot_model.eval()
 rel_model.eval()
 
 device = 'cuda' if torch.cuda.is_available() else 'cpu'
 rel_model = rel_model.to(device)
+chemprot_model = chemprot_model.to(device)
 
 rel_pipe = pipeline(task='text-classification', model=rel_model, tokenizer=rel_tokenizer, device=0 if device=='cuda' else -1)
 
@@ -48,6 +58,10 @@ def download_articles(title: str, start_year: int, end_year: int, max_results: i
         List[Article] A list of articles.
     """
     current_path = Path(os.getcwd()) / 'cache'
+
+    if not os.path.exists(current_path):
+        os.mkdir(current_path)
+
     file_name = '{}_{}_{}_{}.txt'.format(title, start_year, end_year, max_results)
     if os.path.exists(current_path / file_name):
         print("Loading from cache...")
@@ -78,7 +92,6 @@ def download_articles(title: str, start_year: int, end_year: int, max_results: i
     results = Medline.parse(handle)
     results = list(results)
     articles = []
-    print(results[1])
     for article in results:
         abstract = article.get("AB", "?")
         title = article.get("TI", "?")
@@ -87,7 +100,7 @@ def download_articles(title: str, start_year: int, end_year: int, max_results: i
                abstract.find("An amendment to this paper has been published and can be accessed via a link at the top of the paper") == -1 and \
                abstract.find("This corrects the article") == -1 and abstract.find("A Correction to this paaper has been published") == -1:
                 articles.append(Article(title=title, abstract=abstract, 
-                                        pmid=article['PMID'], full_text='', publication_data=datetime.strptime(article['DP'][:4], '%Y')))
+                                        pmid=article['PMID'], full_text='', publication_data=None))
 
     print("Found {} articles".format(len(articles)))
 
@@ -141,7 +154,7 @@ def extract_entities(article: Article, source: str = 'abstract') -> List[Entity]
     annotations = bern_result['annotations']
     entities = []
     for entity in annotations:
-        if (entity['obj'] == 'disease' or entity['obj'] == 'gene'):
+        if (entity['obj'] == 'disease' or entity['obj'] == 'gene' or entity['obj'] == 'drug'):
             pmid = article.pmid
             if type(article.pmid) == list:
                 pmid = article.pmid[0]
@@ -201,25 +214,6 @@ def extract_naive_relations(entities: List[Entity]) -> List[Tuple[Entity, Entity
     return relations
 
 
-# def call_biober_rel(gene_disease_text: Tuple[Entity, Entity, str]):
-#     # TODO: Delete this if we don't use cpu cores
-#     gene_entity, disease_entity, text = gene_disease_text
-
-#     masked_text = ''
-#     if gene_entity.span_begin < disease_entity.span_begin:
-#         masked_text = text[:gene_entity.span_begin] + "@GENE$" + text[gene_entity.span_end:disease_entity.span_begin] + "@DISEASE$" + text[disease_entity.span_end:]
-#     else:
-#         masked_text = text[:disease_entity.span_begin] + "@DISEASE$" + text[disease_entity.span_end:gene_entity.span_begin] + "@GENE$" + text[gene_entity.span_end:]
-    
-#     try:
-#         out = rel_pipe(masked_text)[0]
-#     except:
-#         return (None, None)
-#     if out['label'] == 'LABEL_1':
-#         # TODO: Add the score as the probability
-#         return (gene_entity, disease_entity)
-#     return (None, None)
-
 def extract_biobert_relations(article : Article, source: str = 'abstract', clear_cache: bool = False) -> Tuple[List[Entity], List[Tuple[Entity, Entity]], bool]:
     """
     Extract the entities from the article using BERN2 (online).
@@ -280,14 +274,46 @@ def extract_biobert_relations(article : Article, source: str = 'abstract', clear
     # divide entities in gene and disease entities
     gene_entities = []
     disease_entities = []
+    drug_entities = []
     for entity in entities:
         if entity.type == "disease":
             disease_entities.append(entity)
         elif entity.type == "gene":
             gene_entities.append(entity)
+        elif entity.type == "drug":
+            drug_entities.append(entity)
     
     relations = []
     for gene_entity in gene_entities:
+        # extract gene-drug relations using fine-tuned bert on chemprot
+        for drug_entity in drug_entities:
+            # find the sentence that contains the gene and disease
+            sentence_index_gene = find_entity(gene_entity, span_sentences)
+            sentence_index_drug = find_entity(drug_entity, span_sentences)
+
+            masked_text = ''
+            if gene_entity.span_begin < drug_entity.span_begin:
+                # wrap the gene_entity in << >>
+                # and the drug with [[ ]]
+               masked_text = text[span_sentences[sentence_index_gene][0]:gene_entity.span_begin] + "<< " + text[gene_entity.span_begin:gene_entity.span_end] + " >>" + text[gene_entity.span_end:drug_entity.span_begin] + "[[ " + text[drug_entity.span_begin:drug_entity.span_end] + " ]]" + text[drug_entity.span_end:span_sentences[sentence_index_drug][1]]
+            else:
+                # wrap the drug_entity in << >>
+                # and the gene with [[ ]]
+                masked_text = text[span_sentences[sentence_index_drug][0]:drug_entity.span_begin] + "<< " + text[drug_entity.span_begin:drug_entity.span_end] + " >>" + text[drug_entity.span_end:gene_entity.span_begin] + "[[ " + text[gene_entity.span_begin:gene_entity.span_end] + " ]]" + text[gene_entity.span_end:span_sentences[sentence_index_gene][1]]
+            try:
+                inputs = chemprot_tokenizer(masked_text, return_tensors="pt").to(device)
+                outputs = chemprot_model(**inputs)
+                class_logits = outputs["logits"].detach().cpu().numpy()
+                class_logits = class_logits[0]
+                class_probs = np.exp(class_logits) / np.sum(np.exp(class_logits))
+                class_idx, class_prob = max(enumerate(class_probs), key=lambda x: x[1])
+                if class_prob > 0.5:
+                    relations.append((gene_entity, drug_entity, class_prob))
+            except Exception as e:
+                print(e)
+                continue
+
+        # extract gene-disease relations using biobert
         for disease_entity in disease_entities:
 
             #find the sentence that contains the gene and disease
@@ -406,19 +432,48 @@ def add_title_edge(G, edge):
   result_html += "</ul>"
   return result_html
   
+def _html_graph_communities(G, communities, net):
+    #generate random color for each community
+    colors = ["#"+''.join([random.choice('0123456789ABCDEF') for j in range(6)])
+             for i in range(len(communities))]
+    
+    for i, community in enumerate(communities):
+        for node_id in community:
+            node = net.get_node(node_id)
+            node_type = dict(G.nodes(data=True))[node_id]['type']
+            node['color'] = colors[i]
+            
+            if (node_type == 'gene'):
+                node['shape'] = 'dot'
+            elif (node_type == 'disease'):
+                node['shape'] = 'diamond'
+            elif (node_type == 'drug'):
+                node['shape'] = 'star'
+            
+    
 
-def html_graph(G, name="nodes"):
+def html_graph(G, name="nodes", communities=None, hide_isolated_nodes = True):
+    
+    
+  if hide_isolated_nodes:
+    G.remove_nodes_from(list(nx.isolates(G)))
+    
   net = Network(notebook=True)
 
-  #add nodes
   for n, d in G.nodes(data=True):
     color = '#ccc'
     if d['type'] == 'gene':
       color = '#0DA3E4'
     elif d['type'] == 'disease':
       color = '#bf4d2d'
+    elif d['type'] == 'drug':
+        color = '#5ef951'
     net.add_node(n, d['mention'], color=color, title= add_title_node(G, n, d))
 
+  #if communities is not None and len(communities) > 0 
+  if communities is not None and len(communities) > 0:
+      _html_graph_communities(G, communities, net)
+
   #add edges
   for edge in G.edges(data='True'):
     add_title_edge(G,edge)
